package com.cultural.heritage.service.good.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.CartRecordMapper;
import com.cultural.heritage.mapper.GoodMapper;
import com.cultural.heritage.model.dto.cart.CartRecordAddRequest;
import com.cultural.heritage.model.entity.CartRecord;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.model.vo.CartRecordVO;
import com.cultural.heritage.model.vo.GoodVO;
import com.cultural.heritage.service.good.CartRecordService;
import jakarta.annotation.Resource;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class CartRecordServiceImpl extends ServiceImpl<CartRecordMapper, CartRecord> implements CartRecordService {


    @Resource
    private GoodMapper goodMapper;


    /**
     * 处理用户购物车已存在该商品
     */
    @Override
    public boolean dealAlreadyExistGood(CartRecordAddRequest cartRecordAddRequest) {
        Long userId = cartRecordAddRequest.getUserId();
        Long goodId = cartRecordAddRequest.getGoodId();
        QueryWrapper<CartRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        queryWrapper.eq("goodId", goodId);
        CartRecord cartRecord = this.baseMapper.selectOne(queryWrapper);
        if (cartRecord != null) {
            cartRecord.setQuantity(cartRecord.getQuantity() + cartRecordAddRequest.getQuantity());
            cartRecord.setSubtotal(cartRecord.getSubtotal() + cartRecordAddRequest.getSubtotal());
            boolean result = this.updateById(cartRecord);
            ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        } else {
            BeanUtils.copyProperties(cartRecordAddRequest, cartRecord);
            boolean result = this.save(cartRecord);
            ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        }
        return true;
    }



    /**
     * 封装用户购物车中的商品信息
     */
    @Override
    public List<CartRecordVO> transformToCartRecordVOList(List<CartRecord> list) {
        return list.stream().map(cartRecord -> {
            // 封装购物车记录VO
            CartRecordVO cartRecordVO = new CartRecordVO();
            BeanUtils.copyProperties(cartRecord, cartRecordVO);

            // 封装商品类GoodVO
            Long goodId = cartRecord.getGoodId();
            QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("id", goodId);
            Good good = goodMapper.selectOne(queryWrapper);
            GoodVO goodVO = new GoodVO();
            BeanUtils.copyProperties(good, goodVO);

            cartRecordVO.setGoodVO(goodVO);
            return cartRecordVO;
        }).toList();
    }
}
